/* -*- c-basic-offset: 4 -*- */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include "net.h"
#include "game.h"
#include "asciiart.h"

#define MAX_MSG_LEN 50
typedef enum { MSG_MOVE, MSG_ROTATE } move_t;

typedef struct {
    move_t type;
    union {
	struct {
	    int row, col;
	    dir_t dir;
	} move;
	struct {
	    int row, col;
	    rot_t dir;
	} rotate;
    } args;
} message_t;


// FIXME rewrite this to format the board in a char *?
/*
void
print_board_small()
{
    int x, y;

    char vline, hline, xline, lline;
    char dead_piece[] = "##";
    vline = '|', hline = '-', xline = '+';
    lline = '#';

    net_all_printf("%s(%c)\n", sideinfo[RED].name, sideinfo[RED].ch);

    // column numbers
    net_all_printf("  %c", vline);
    for (x = 0; x < BOARD_WIDTH; x++) {
	net_all_printf(" %d%c", x, vline);
    }
    net_all_printf("\n");

    // line
    net_all_printf("%c%c%c", hline, hline, xline);
    for (x = 0; x < BOARD_WIDTH; x++)
	net_all_printf("%c%c%c",
	       laser_at(x, -1) ? lline : hline,
	       laser_at(x, -1) ? lline : hline,
	       xline);
    net_all_printf("%c%c\n", hline, hline);

    for (y = 0; y < BOARD_HEIGHT; y++) {
	// row name
	net_all_printf(" %c%c", 'A'+y, laser_at(-1,y) ? lline : vline);
	// row
	for (x = 0; x < BOARD_WIDTH; x++) {
	    if (laser_at(x, y))
		net_all_printf("%s", dead_piece);
	    else
		print_piece_small(board[x][y].piece,
				  board[x][y].side,
				  board[x][y].dir);
	    net_all_printf("%c", (x==BOARD_WIDTH-1 && laser_at(x+1, y)) ? lline : vline);
	}
	// row name
	net_all_printf("%c\n", 'A'+y);

	// line
	net_all_printf("%c%c%c", hline, hline, xline);
	for (x = 0; x < BOARD_WIDTH; x++)
	    net_all_printf("%c%c%c",
		   (y==BOARD_HEIGHT-1 && laser_at(x, y+1)) ? lline : hline,
		   (y==BOARD_HEIGHT-1 && laser_at(x, y+1)) ? lline : hline,
		   xline);
	net_all_printf("%c%c\n", hline, hline);
    }

    // column numbers
    net_all_printf("  %c", vline);
    for (x = 0; x < BOARD_WIDTH; x++)
	net_all_printf(" %d%c", x, vline);
    net_all_printf("\n");

    for (x = 0; x < (BOARD_WIDTH+2)*3-1-strlen(sideinfo[SILVER].name)-3; x++)
	net_all_printf(" ");
    net_all_printf("%s(%c)\n", sideinfo[SILVER].name, sideinfo[SILVER].ch);
}
*/

int
read_message(int side, message_t *m)
{
    char buf[MAX_MSG_LEN+1];
    char c;
    int i;
    for (i = 0; i < MAX_MSG_LEN+1; i++) {
	c = net_getc(side);
	printf("input '%c'\n", c);
	if (c == '\n')
	    break;
	buf[i] = c;
    }
    buf[i] = '\0';
    while (c != '\n') c = net_getc(side);

    switch (buf[0]) {
    case 'M': m->type = MSG_MOVE; break;
    case 'R': m->type = MSG_ROTATE; break;
    default: return E_UNKNOWN_COMMAND;
    }

    if (m->type == MSG_MOVE) {
	m->args.move.row = buf[2]-'A';
	m->args.move.col = buf[3]-'0';
	switch (buf[5]) {
	case 'N':
	    m->args.move.dir = (buf[6]=='W' ? D_NW : (buf[6]=='E' ? D_NE : D_N));
	    break;
	case 'S':
	    m->args.move.dir = (buf[6]=='W' ? D_SW : (buf[6]=='E' ? D_SE : D_S));
	    break;
	case 'E':
	    m->args.move.dir = D_E;
	    break;
	case 'W':
	    m->args.move.dir = D_W;
	    break;
	}

	net_all_printf("msg: MOVE (%d,%d) %d\n", m->args.move.row, m->args.move.col,
		       m->args.move.dir);
    } else if (m->type == MSG_ROTATE) {
	m->args.rotate.row = buf[2]-'A';
	m->args.rotate.col = buf[3]-'0';
	m->args.rotate.dir = (buf[5]=='L' ? R_CCW : R_CW);
    }

    return HUGE_SUCCESS;
}

int
execute_message(game_t *game, message_t msg, int side)
{
    switch (msg.type) {
    case MSG_MOVE:
	return move(game, side,
		    msg.args.move.row, msg.args.move.col,
		    msg.args.move.dir, 0);
    case MSG_ROTATE:
	return rotate(game, side,
		      msg.args.rotate.row, msg.args.rotate.col,
		      msg.args.rotate.dir);
    default:
	net_side_printf(side, "unknown command type %d\n", msg.type);
	return E_UNKNOWN_COMMAND;
    }
}

void
simple_game(void)
{
    int side = 0;

    game_t game;
    char aabuf[ASCIIART_SMALL_BUFSZ];

    init_game(&game);
    setup_board(&game, classic_setup);
    asciiart_small_draw_board(aabuf, &game);
    net_all_printf("%s", aabuf);
    //print_board_small();

    for (;;) {
	message_t msg;
	int r;

	net_all_printf("%s's move\n", sideinfo[side].name);
    prompt:
	net_side_printf(side, "> ");
	net_all_flush();
	
	r = read_message(side, &msg);
	switch (r) {
	case HUGE_SUCCESS: break;
	case E_UNKNOWN_COMMAND:
	    net_side_printf(side, "unknown command\n");
	    goto prompt;
	default:
	    net_side_printf(side, "uh oh (strange return value from read_message: %d)\n", r);
	    goto prompt;
	}

	r = execute_message(&game, msg, side);
	switch (r) {
	case HUGE_SUCCESS:
	    //print_board_small();
	    asciiart_small_draw_board(aabuf, &game);
	    net_all_printf("%s", aabuf);
	    r = laser(&game, side);
	    //print_board_small();
	    asciiart_small_draw_board(aabuf, &game);
	    net_all_printf("%s", aabuf);
	    switch (r) {
	    case SILVER_WINS:
		net_all_printf("SILVER WINS\n");
		return;
	    case RED_WINS:
		net_all_printf("RED WINS\n");
		return;
	    case L_WALL:
	    case L_PIECE:
		net_all_printf("laser hits %s at %c%c\n",
			       r==L_WALL ? "wall" : "piece",
			       game.laser_pos[1]+'A', game.laser_pos[0]+'0');
		break;
	    }
	    side = 1-side;
	    break;
	case E_ILLEGAL_MOVE:
	    net_side_printf(side, "illegal move\n");
	    goto prompt;
	default:
	    net_side_printf(side, "uh oh (strange return value from execute_message: %d)\n", r);
	    goto prompt;
	}
    }
}

int
main() {
    net_server(15239);
    simple_game();
}
